import openpyxl

from InterfaceAutoTest_V1.common import log
from InterfaceAutoTest_V1.common.read_ini import ReadIni
from InterfaceAutoTest_V1.common.read_json import read_json


class ReadExcel:
    def __init__(self, table_name: str):
        self.read_ini = ReadIni()
        try:
            self.wb = openpyxl.load_workbook(self.read_ini.get_file_path('excel'))
            self.ws = self.wb[table_name]
        except:
            log.error('获取工作表失败，请察看工作表名称是否配置正确！！！')
        self.case_data_dict = read_json(self.read_ini.get_file_path('case'))
        self.expect_data_dict = read_json(self.read_ini.get_file_path('expect'))
        self.sql_data_dict = read_json(self.read_ini.get_file_path('sql'))

    def __get_cell_value(self, column: str, row: int) -> str:
        try:
            value = self.ws[column + str(row)].value
            if value is None:
                return None
            elif value.strip():
                return value.strip()
        except:
            log.error('获取指定单元格数据失败,请检查输入的列号和行号是否正确！！')

    def module_name(self, row):
        return self.__get_cell_value('b', row)

    def api_name(self, row):
        return self.__get_cell_value('c', row)

    def case_title(self, row):
        return self.__get_cell_value('d', row)

    def case_level(self, row):
        return self.__get_cell_value('e', row)

    def case_url(self, row):
        return self.read_ini.get_host_value('host') + self.__get_cell_value('g', row)

    def case_method(self, row):
        return self.__get_cell_value('f', row)

    def case_mime(self, row):
        mime = self.__get_cell_value('h', row)
        if mime:
            return mime.lower()

    def case_data(self, row):
        case_data_key = self.__get_cell_value('i', row)
        if case_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            try:
                return self.case_data_dict[module_name][api_name][case_data_key]
            except:
                log.error('请检查用例数据的配置！！！')

    def expect_data(self, row):
        expect_data_key = self.__get_cell_value('j', row)
        if expect_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            try:
                return self.expect_data_dict[module_name][api_name][expect_data_key]
            except:
                log.error('请检查期望数据的配置！！！')

    def sql_type(self, row):
        value = self.__get_cell_value('k', row)
        if value:
            return value.lower()

    def sql_data(self, row):
        sql_data_key = self.__get_cell_value('l', row)
        if sql_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            try:
                return self.sql_data_dict[module_name][api_name][sql_data_key]
            except:
                log.error('请检查sql语句数据的配置！！！')

    def update_key(self, row):
        return self.__get_cell_value('m', row)

    def get_data(self):
        data_list = []
        try:
            for row in range(2, self.ws.max_row + 1):
                module_name =self.module_name(row)
                api_name = self.api_name(row)
                case_title = self.case_title(row)
                case_level = self.case_level(row)
                case_url = self.case_url(row)
                case_method = self.case_method(row)
                case_mime = self.case_mime(row)
                case_data = self.case_data(row)
                expect_data = self.expect_data(row)
                sql_type = self.sql_type(row)
                sql_data = self.sql_data(row)
                update_key = self.update_key(row)
                data_list.append([module_name,api_name,case_title,case_level,case_url, case_method, case_mime,  case_data, expect_data, sql_type,
                                  sql_data, update_key])
            else:
                return data_list
        except:
            log.error('请检查excel中是否存在空行,如果存在,请删除空行！！')


print(ReadExcel('BPM-WDF').get_data())
